/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 *
 * hal_LCD.c
 *
 * Hardware abstraction layer for the FH-1138P Segmented LCD
 * on MSP-EXP430FR6989
 *
 * February 2015
 * E. Chen
 *
 ******************************************************************************
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019
 * (c) Pascal Chour
 ******************************************************************************/


#include <hal_LCD.h>
#include "driverlib/MSP430FR5xx_6xx/driverlib.h"
#include "string.h"

volatile unsigned char mode;

const uint8_t Posit[] = {7,14,18,3,5,9};

// LCD memory map for numeric digits
const char digit[10][2] =
{
    {0xFC, 0x28},  /* "0" LCD segments a+b+c+d+e+f+k+q */
    {0x0C, 0x08},  /* "1" */
    {0xDB, 0x00},  /* "2" */
    {0x9F, 0x00},  /* "3" */
    {0x2F, 0x00},  /* "4" */
    {0xB7, 0x00},  /* "5" */
    {0xF7, 0x00},  /* "6" */
    {0x3C, 0x00},  /* "7" */
    {0xFF, 0x00},  /* "8" */
    {0xBF, 0x00}   /* "9" */
};

// LCD memory map for uppercase letters
const char alphabetBig[26][2] =
{
    {0x7F, 0x00},  /* "A" LCD segments b+c+d+e+f+g+m */
    {0x9E, 0x50},  /* "B" */
    {0xF0, 0x00},  /* "C" */
    {0x9C, 0x50},  /* "D" */
    {0xF3, 0x00},  /* "E" */
    {0x73, 0x00},  /* "F" */
    {0xF6, 0x00},  /* "G" */
    {0x6F, 0x00},  /* "H" */
    {0x90, 0x50},  /* "I" */
    {0xCC, 0x00},  /* "J" */
    {0x61, 0x88},  /* "K" */
    {0xE0, 0x00},  /* "L" */
    {0x6C, 0x0A},  /* "M" */
    {0x6C, 0x82},  /* "N" */
    {0xFC, 0x00},  /* "O" */
    {0x7B, 0x00},  /* "P" */
    {0xFC, 0x80},  /* "Q" */
    {0x7B, 0x80},  /* "R" */
    {0xB7, 0x00},  /* "S" */
    {0x10, 0x50},  /* "T" */
    {0xEC, 0x00},  /* "U" */
    {0x60, 0x28},  /* "V" */
    {0x6C, 0xA0},  /* "W" */
    {0x00, 0xAA},  /* "X" */
    {0x00, 0x4A},  /* "Y" */
    {0x90, 0x28}   /* "Z" */
};

void LCD_init()
{
    LCD_C_initParam initParams = {0};
    initParams.clockSource = LCD_C_CLOCKSOURCE_ACLK;
    initParams.clockDivider = LCD_C_CLOCKDIVIDER_1;
    initParams.clockPrescalar = LCD_C_CLOCKPRESCALAR_16;
    initParams.muxRate = LCD_C_4_MUX;
    initParams.waveforms = LCD_C_LOW_POWER_WAVEFORMS;
    initParams.segments = LCD_C_SEGMENTS_ENABLED;

    LCD_C_init(LCD_C_BASE, &initParams);
    // LCD Operation - VLCD generated internally, V2-V4 generated internally, v5 to ground

    /*  'FR6989 LaunchPad LCD1 uses Segments S4, S6-S21, S27-S31 and S35-S39 */
    LCD_C_setPinAsLCDFunctionEx(LCD_C_BASE, LCD_C_SEGMENT_LINE_4, LCD_C_SEGMENT_LINE_4);
    LCD_C_setPinAsLCDFunctionEx(LCD_C_BASE, LCD_C_SEGMENT_LINE_6, LCD_C_SEGMENT_LINE_21);
    LCD_C_setPinAsLCDFunctionEx(LCD_C_BASE, LCD_C_SEGMENT_LINE_27,LCD_C_SEGMENT_LINE_31);
    LCD_C_setPinAsLCDFunctionEx(LCD_C_BASE, LCD_C_SEGMENT_LINE_35, LCD_C_SEGMENT_LINE_39);


// VLCD gnr en externe (ne fonctionne pas avec cette configuration)
#ifdef VLCD_EXTERNE
    LCDCCTL0 &= ~LCDON;
    // VLCD = 0 (charge pump disabled), LCDREXT = 0 (bias gnr en interne), R03EXT = 0 (V5 = Vss), LCDEXTBIAS = 0 (BIAS gnr en interne), VLCDEXT = 1 (VLCD = Vcc)
    // LCDCPEN = 0 (Charge Pump Disabled), VLCDREF = 0 (??? Internal reference voltage), LCD2B = 0 (Static BIAS = 1/3)
    LCDCVCTL = 0x0012;
//    LCD_C_configChargePump(LCD_C_BASE, LCD_C_SYNCHRONIZATION_ENABLED, 0);

#else
    LCD_C_setVLCDSource(LCD_C_BASE, LCD_C_VLCD_GENERATED_INTERNALLY, LCD_C_V2V3V4_GENERATED_INTERNALLY_NOT_SWITCHED_TO_PINS,  LCD_C_V5_VSS);
    //Set VLCD voltage to 3.20v
    LCD_C_setVLCDVoltage(LCD_C_BASE, LCD_C_CHARGEPUMP_VOLTAGE_3_02V_OR_2_52VREF);

    // Enable charge pump and select internal reference for it
    LCD_C_enableChargePump(LCD_C_BASE);
    LCD_C_selectChargePumpReference(LCD_C_BASE, LCD_C_INTERNAL_REFERENCE_VOLTAGE);
    LCD_C_configChargePump(LCD_C_BASE, LCD_C_SYNCHRONIZATION_ENABLED, 0);
#endif

    LCDCBLKCTL |= LCDBLKPRE1 | LCDBLKPRE2 | LCDBLKMOD_1;

    // Clear LCD memory
    LCD_C_clearMemory(LCD_C_BASE);

    //Turn LCD on
    LCD_C_on(LCD_C_BASE);
}

/*
 * Scrolls input string across LCD screen from left to right
 */
void displayScrollText(const char *msg)
{
    uint16_t length = strlen(msg);
    uint16_t i = 0;
    int16_t j;
    char buffer[6];
    while (i < length+1) {
        for (j = 0; j < 6; j++) {
            if ((i+j) < length) {
                buffer[j] = msg[i+j];
            } else {
                buffer[j] = ' ';
            }
        }
        for (j=0; j<6; j++) {showChar(buffer[j], j+1);}
        __delay_cycles(1000000);
        i = i+1;
    }
}

/*
 * Displays input character at given LCD digit/position
 * Only spaces, numeric digits, and uppercase letters are accepted characters
 */
void showChar(char c, uint8_t position)
{
    uint8_t vl = 0xFF;
    uint8_t vh = 0xFF;
    if (c == 0x00) return;
    if ((position < 1) || (position > 6)) return;
    if ((c == ' ') || (c == ','))
    {
        vl = 0;
        vh = 0;
    }
    else if (c == '=')
    {
        vl = 0x83;
        vh = 0x00;
    }
    else if (c >= '0' && c <= '9')
    {
        vl = digit[c-48][0];
        vh = digit[c-48][1];
    }
    else if (c >= 'A' && c <= 'Z')
    {
        vl = alphabetBig[c-65][0];
        vh = alphabetBig[c-65][1];
    }
    else if (c == '-')
    {
        vl = 0x03;
        vh = 0x00;
    }
    else if (c == '.')
    {
        vl = 0x00;
        vh = 0x20;
    }
    else if (c == '+')
    {
        vl = 0x03;
        vh = 0x50;
    }
    else if (c == '}') // ->
    {
        vl = 0x01;
        vh = 0x22;
    }
    position = Posit[position-1];
    LCDMEM[position] = vl;         // Display  char
    LCDMEM[position+1] = vh;
}

/*
 * Clears memories to all 6 digits on the LCD
 */
void LCD_clear()
{
    LCD_ClearText();
    LCDM14 = LCDBM14 = 0x00;
    LCDM18 = LCDBM18 = 0x00;
    LCDM3 = LCDBM3 = 0x00;
}

/*
 * Efface les digits chiffres/textes et arrte le clignotement
 */
void LCD_ClearText()
{
    uint16_t i;
    for (i=0; i < 6; i++) {
        LCDMEM[Posit[i]] = LCDBMEM[Posit[i]] = 0;
        LCDMEM[Posit[i]+1] = LCDBMEM[Posit[i]+1] = 0;
    }
}


// Affiche (True) / efface (False) Antenne en fonction de OnOff
void LCD_displayAnt(bool OnOff, bool clignote)
{
    if (OnOff) {LCDMEM[4] |= 0x04;} else {LCDMEM[4] &= 0xFB;}
    if (clignote) {LCDBMEM[4] |= 0x04;} else {LCDBMEM[4] &= 0xFB;}
}


// Affiche (True) / efface (False) Timer en fonction de OnOff
void LCD_displayTmr(bool OnOff, bool clignote)
{
    if (OnOff) {LCDMEM[2] |= 0x08;} else {LCDMEM[2] &= 0xF7;}
    if (clignote) {LCDBMEM[2] |= 0x08;} else {LCDBMEM[2] &= 0xF7;}
}

// Affiche (True) / efface (False) ! en fonction de OnOff
void LCD_displayAlarm(bool OnOff, bool clignote)
{
    if (OnOff) {LCDMEM[2] |= 0x01;} else {LCDMEM[2] &= 0xFE;}
    if (clignote) {LCDBMEM[2] |= 0x01;} else {LCDBMEM[2] &= 0xFE;}
}

// Affiche (True) / efface (False) Coeur en fonction de OnOff
void LCD_displayHeart(bool OnOff, bool clignote)
{
    if (OnOff) {LCDMEM[2] |=  0x04;} else {LCDMEM[2] &= 0xFB;}
    if (clignote) {LCDBMEM[2] |=  0x04;} else {LCDBMEM[2] &= 0xFB;}
}

// Affiche (True) / efface (False) Coeur en fonction de OnOff
void LCD_displayRXTX(bool OnOff, bool clignote)
{
    if (OnOff) {LCDMEM[8] |=  0x05;} else {LCDMEM[8] &= 0xFA;}
    if (clignote) {LCDBMEM[8] |=  0x05;} else {LCDBMEM[8] &= 0xFA;}
}

// Affiche (OnOff True) / efface (OnOff False) Niveau de batterie. Level = 0  6
// Les barres droites et gauche ne cligntent jamais
void LCD_displayLevel(unsigned int level, bool OnOff, bool clignote)
{
    if (OnOff) {
        LCDMEM[13] = 0x00; // efface batterie
        LCDMEM[17] = 0x10; // affiche les []
        LCDBMEM[13] = 0x00;
        LCDBMEM[17] = 0x00;
        if (level > 6) {level = 6;}
        switch (level)
        {
            case 6 : LCDMEM[17] |= 0x20; // B1
            case 5 : LCDMEM[13] |= 0x20; // B2
            case 4 : LCDMEM[17] |= 0x40; // B3
            case 3 : LCDMEM[13] |= 0x40; // B4
            case 2 : LCDMEM[17] |= 0x80; // B5
            case 1 : LCDMEM[13] |= 0x80; // B6
        }
        if (clignote) {
            LCDBMEM[13] |= 0xE0;
            LCDBMEM[17] |= 0xE0;
        }
    } else {
        LCDMEM[13] &= 0x0F; // efface tout
        LCDMEM[17] &= 0x0F;
        LCDBMEM[13] &= 0x0F;
        LCDBMEM[17] &= 0x0F;
    }
}


//
// Ecrit un nombre cadr  droite de l'afficheur
// En entre : number = nombre  crire
//
void LCD_writeIntRight (uint32_t number)  // A cool function that moves number right to left
{
   int i;
   if (number > 999999) { // si dpassement
       for (i = 6; i > 0; i--) {showChar('*',i);}      // On rempli avec des toiles
   } else {
       i = 1;
       do      // Extract each digit in number, put in an integer array, and count total length also
       {
          showChar((number % 10) | 0x30,7-i);
          i = i+1;
          number = number / 10;     // remove the least significant character from number
       } while (number > 0);
   }
}

//
// Ecrit un texte cadr  gauche de l'afficheur
// msg = texte  afficher
// lg = longueur du texte (tronqu  la taille max de l'afficheur par la fonction
//
void LCD_writeTxtLeft (const char *buf, bool Efface)
{
    int i;
    i = 0;
    while ((i < 6) && (buf[i] != 0)) {
        showChar(buf[i], i+1);
        i = i+1;
    }
    while ((i < 6) && (Efface)) {
        showChar(' ', i+1);
        i = i+1;
    }
}

//
// Fait clignoter ou non la zone de texte
void LCD_clignoteTxt(bool clignote)
{
    uint8_t i;
    for (i = 0; i < 6; i++) {
        if (clignote) {
            LCDBMEM[Posit[i]] = 0xFF;
            LCDBMEM[Posit[i]+1] = 0xFF;
        } else {
            LCDBMEM[Posit[i]] = 0x00;
            LCDBMEM[Posit[i]+1] = 0x00;
        }
    }
}
